///////////////////////////////////////////////////
// AGXDigitalIo.cpp
// 720020-1171x
// 720020-1171x
//
// Applied Data Systems
//
// This simple example shows how to read and write data on the seven
// exposed digital GPIOs of the ADS AGX device from the CPLD.
// 
//
// History
// -------
// 720020-11711: February 9, 2004 (ctacke)
//  - Initial implementation//
///////////////////////////////////////////////////
#include <windows.h>

#define	MEMORY_SIZE					0x10000  // 64kiB memory space; max allocation for VirtualAlloc is 64MiB
                                     // must be 64kiB aligned (VirtualAlloc() requirement)
#define AGX_CPLD_GPIO_BASE_ADDR		0x08040000
#define AGX_GPIO_DIR_OFFSET			0
#define AGX_GPIO_DATA_OFFSET		4

// VirtualCopy definition
extern "C" BOOL VirtualCopy(LPVOID lpvDestMem, LPVOID lpvSrcMem, 
							DWORD dwSizeInBytes, DWORD dwProtectFlag);

LPVOID MapAddress(DWORD baseAddress);
void UnmapAddress(LPVOID address);

int WINAPI WinMain(	HINSTANCE hInstance,
				   HINSTANCE hPrevInstance,
				   LPTSTR    lpCmdLine,
				   int       nCmdShow)
{
	int				result;
	DWORD			*pGPIO;

	_tprintf(_T("<< Starting AGX Digital IO Test >>\r\n"));

	// map the physical addresses for the direction and data
	pGPIO = (DWORD*)MapAddress(AGX_CPLD_GPIO_BASE_ADDR);

	if(pGPIO == NULL)
	{
		_tprintf(_T("Failed to map GPIO memory registers with error %i\r\n"), GetLastError());
		result = _tscanf(_T("<< AGX Digital IO Test Complete>>"));

		return -1;
	}

	// set all lines as output
	_tprintf(_T("Setting CPLD IOs as outputs...\r\n"));

	// the GPIOs use only the 7 LSBs of the register
	pGPIO[AGX_GPIO_DIR_OFFSET / sizeof(DWORD)] = 0x7F;

	Sleep(50);

	// write a walking one on the CPLD pins
	for(int i = 0 ; i < 7 ; i++)
	{
		_tprintf(_T("Pulsing CPLD IO line %i\r\n"), i);

		// set the line high
		pGPIO[AGX_GPIO_DATA_OFFSET / sizeof(DWORD)] = (1 << i);
		Sleep(500);

		// set it low
		pGPIO[AGX_GPIO_DATA_OFFSET / sizeof(DWORD)] = 0;
		Sleep(500);
	}


	// change the direction to all inputs
	_tprintf(_T("Setting CPLD IOs as outputs...\r\n"));

	// the GPIOs use only the 7 LSBs of the register
	pGPIO[AGX_GPIO_DIR_OFFSET / sizeof(DWORD)] = 0x00F;


	// read the state
	DWORD state = pGPIO[AGX_GPIO_DATA_OFFSET / sizeof(DWORD)] & 0x7F;

	// report the state
	_tprintf(_T("CPLD IO line states are 0x%02x\r\n"), state);

	// free resources
	UnmapAddress(pGPIO);

	result = _tscanf(_T("<< AGX Digital IO Test Complete>>"));

	return 0;
}


/////////////////////////////////////////////
// MapAddress 
//  Maps the desired memory address.
//  This function should be called first and its
//  return value is then passed to the ReadData16 or
//  WriteData16 functions as the baseAddress.
//
// [I] baseAddress - base address to map
// [O] Virtual Address pointer to mapped region
// 
/////////////////////////////////////////////
LPVOID MapAddress(DWORD baseAddress)
{
	LPVOID pAddr = VirtualAlloc(0, MEMORY_SIZE, MEM_RESERVE, PAGE_NOACCESS);
	
	if(pAddr)	
	{
		if(VirtualCopy (pAddr, (PVOID)(baseAddress >> 8), 
				MEMORY_SIZE, PAGE_PHYSICAL|PAGE_READWRITE|PAGE_NOCACHE) == TRUE) 
		{
				DEBUGMSG(1, (_T("0x%08x mapped to: 0x%08x\r\n"), baseAddress, pAddr));
		}
		else 
		{
			RETAILMSG(1, (_T("Error! at VirtualCopy()\r\n")));

			return NULL;
		}
	}
	else 
	{
		RETAILMSG(1, (_T("Error! at VirtualAlloc()\r\n")));
		return NULL;
	}

	return pAddr;
}

void UnmapAddress(LPVOID address)
{
	VirtualFree(address, 0, MEM_RELEASE);
	address = NULL;
}
